/*********************************************************************************
 *
 * File name:		defination_optical.h
 * Version:			4.0
 * Software:
 * Authors:			Zhehui Wang, Shixi Chen, Jiang Xu
 * Website:			https://eexu.home.ece.ust.hk/OEIL.html
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/
#ifndef LASERPOWERBACKTRACER_H
#define LASERPOWERBACKTRACER_H

#include "NonlinearLossModel.h"
#include <boost/numeric/odeint.hpp>

/**
 * @brief LaserPowerBacktracer does exactly the reverse of the NonlinearLossModel which it inherits from. It intakes the final
 * power (in Watt) and calculates the initial coupled-in power and the corresponding propagation loss.
 */
class LaserPowerBacktracer : public NonlinearLossModel {
  private:
    double final_power_;
    double final_intensity_;

  public:
#ifdef __DEBUG_
    LaserPowerBacktracer();
#endif
    LaserPowerBacktracer(double powerInWattBeforeReceiver, double waveguideLength);

    double getFinalPower() const { return final_power_; }

    /**
     * @brief  This function sets the required coupled-out power in Watt.
     * @param finalPower The final power in Watt.
     */
    void setFinalPower(double finalPower);
    void updateModel();

    void echoParameters() const;
    double totalLossAcrossWaveguide();
    double nonlinearLossAcrossWaveguide();

    virtual void operator()(const intensity_state& I, intensity_state& dIdz, const double z) const override;
};
#endif