/*********************************************************************************
 *
 * File name:		defination_electrical.h
 * Version:			4.0
 * Software:
 * Authors:			Zhehui Wang, Shixi Chen, Jiang Xu
 * Website:			https://eexu.home.ece.ust.hk/OEIL.html
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/

#ifndef DEFINITION_ELECTRICAL_H
#define DEFINITION_ELECTRICAL_H

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <fstream>

/*file pointer to the files*/
FILE* parameter_flie_electrical;
FILE* configuration_file_electrical;
FILE* output_file_electrical;

/*The following is for PCB*/
double pcb_layer_height;     // The height of a single PCB layer unit:mil
double pcb_trace_width;      // The width of a PCB traces unit:mil
double pcb_trace_height;     // The height of a PCB traces unit:mil
double pcb_trace_pair_pitch; // The pitch of PCB trace pairs unit:mil
double pcb_loss_tangent;     // The loss tangent of dielectric material of PCB board unit:N/A
double pcb_dielectric;       // The relative dielectric constant value of PCB board unit:N/A
double package_pin_pitch;    // The pitch of electrical pins on the package :mm

/*The following is for trace*/
double trace_half_depth_f;     // The frequency when the skin depth is equal to half of trace height unit:MHz
double trace_characteristic_z; // The xxx unit:mOhm
double trace_unit_length_c;    // The xxx unit:pF/cm
double trace_direct_current_r; // The xxx unit:mOhm
double trace_input_impendance; // The xxx unit:mOhm
double electrical_pin_load_c;  // The xxx unit:pF

/*The following is for transceiver*/
double la_threshold_voltage; // The threshold voltage of the limiting amplifier unit:mV
double la_offset_coefficent; // The offset coefficent of the limiting amplifier unit:N/A
double la_coefficent_margin; // The coefficent margin of the limiting amplifier unit:N/A
double circuit_voltage;      // The supply voltage of the circuit unit:V

/*The following is for serdes*/
double serdes_cur_electrical;  // The unit current in SerDes circuit unit in electrical interconnect unit:mA/Gbps
double serdes_area_electrical; // The unit area in SerDes circuit unit in electrical interconnect unit:um^2/Gbps

/*The following is for configurations*/
double
    data_rate_electrical; // The data rate of signals in electrical interconnects, which is double the working frequency unit:N/A
double length_electrical; // the length of the electrical interconnect unit:cm
int serdes_ratio_electrical;   // the parallel-to-serial ratio of SerDes in electrical interconnect unit:N/A
int number_of_parallel_traces; // number of PCB trace pairs in electrical interconnects unit:N/A

/*The following is for calculated results*/
double sensitivity_la;                   // the sensitivity of electrical receiver unit:mV
double crosstalk_coefficient_electrical; // the ratio of noise voltage to signal voltage unit:N/A
double total_attenuation_electrical;     // the total attenuation on the entire electrical interconenct unit: N/A
double energy_consumption_electrical;    // the energy consumed by interconect per bit unit:pJ/bit
double area_density_electrical;          // the area bandwidth density of electrical interconnect unit:Gbps/mm^2
double linear_density_electrical;        // the linear bandwidth density of electrical interconnect unit:Gbps/mm
double area_electrical;                  // the area of electrical interconnect unit:mm^2
double latency_value_electrical;         // the signal latency of electrical interconnect unit:ns

/*this is to read the paramters from the file parameter_electrical.txt*/
void read_parameter_electrical() {
    printf("Electrical:\n");
    char* name = new char[50];
    char* unit = new char[20];
    parameter_flie_electrical = fopen("../config/parameter_electrical.txt", "r");
    if (parameter_flie_electrical == NULL) {
        printf("Cannot open file parameter_electrical.txt\n");
        exit(-1);
    }
    /*skip the text file head.*/
    for (int i = 0; i < 10; i++) {
        char* line = NULL;
        size_t len = 0;
        getline(&line, &len, parameter_flie_electrical);
    }
    // for (int i = 0; i < 37; i++) {
    //     fscanf(parameter_flie_electrical, "%s\n", name);
    // }
    /*parameters for PCB*/
    fscanf(parameter_flie_electrical, "%s\n", name);
    printf("%s\n", name);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &pcb_layer_height, name, unit);
    printf("%s\t%f\t%s\n", name, pcb_layer_height, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &pcb_trace_width, name, unit);
    printf("%s\t\t%f\t%s\n", name, pcb_trace_width, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &pcb_trace_height, name, unit);
    printf("%s\t%f\t%s\n", name, pcb_trace_height, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &pcb_trace_pair_pitch, name, unit);
    printf("%s\t%f\t%s\n", name, pcb_trace_pair_pitch, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &pcb_loss_tangent, name, unit);
    printf("%s\t%f\t%s\n", name, pcb_loss_tangent, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &pcb_dielectric, name, unit);
    printf("%s\t\t%f\t%s\n", name, pcb_dielectric, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &package_pin_pitch, name, unit);
    printf("%s\t%f\t%s\n", name, package_pin_pitch, unit);

    /*parameters for trace*/
    fscanf(parameter_flie_electrical, "%s\n", name);
    printf("%s\n", name);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &trace_half_depth_f, name, unit);
    printf("%s\t%f\t%s\n", name, trace_half_depth_f, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &trace_characteristic_z, name, unit);
    printf("%s\t%f\t%s\n", name, trace_characteristic_z, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &trace_unit_length_c, name, unit);
    printf("%s\t%f\t%s\n", name, trace_unit_length_c, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &trace_direct_current_r, name, unit);
    printf("%s\t%f\t%s\n", name, trace_direct_current_r, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &trace_input_impendance, name, unit);
    printf("%s\t%f\t%s\n", name, trace_input_impendance, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &electrical_pin_load_c, name, unit);
    printf("%s\t%f\t%s\n", name, electrical_pin_load_c, unit);

    /*parameters for transceiver*/
    fscanf(parameter_flie_electrical, "%s\n", name);
    printf("%s\n", name);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &la_threshold_voltage, name, unit);
    printf("%s\t%f\t%s\n", name, la_threshold_voltage, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &la_offset_coefficent, name, unit);
    printf("%s\t%f\t%s\n", name, la_offset_coefficent, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &la_coefficent_margin, name, unit);
    printf("%s\t%f\t%s\n", name, la_coefficent_margin, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &circuit_voltage, name, unit);
    printf("%s\t\t%f\t%s\n", name, circuit_voltage, unit);

    /*parameters for serdes*/
    fscanf(parameter_flie_electrical, "%s\n", name);
    printf("%s\n", name);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &serdes_cur_electrical, name, unit);
    printf("%s\t%f\t%s\n", name, serdes_cur_electrical, unit);
    fscanf(parameter_flie_electrical, "%lf %s %s\n", &serdes_area_electrical, name, unit);
    printf("%s\t%f\t%s\n", name, serdes_area_electrical, unit);
    printf("\n");

    fclose(parameter_flie_electrical);
}

/*this is to read the configuration from the file configuration_electrical.txt*/
void read_configuration_electrical() {
    char name[50];
    char unit[20];

    // A really wired BUG here. Still don't know why this doean't work
    // Segfault is thrown with the file handler non-null
    // configuration_file_electrical = fopen("configuration_electrical.txt", "r");
    // if (configuration_file_electrical == NULL) {
    //     printf("Cannot open file configuration_electrical.txt\n");
    // }
    // fscanf(configuration_file_electrical, "%s\n", name);
    // printf("A word %s is read.", name);

    using namespace std;
    ifstream parameter_flie_electrical("../config/configuration_electrical.txt");

    /*skip the text file head.*/
    for (int i = 0; i < 10; i++) {
        string line;
        getline(parameter_flie_electrical, line);
    }

    // for (int i = 0; i < 37; i++) {
    //     fscanf(configuration_file_electrical, "%s\n", name);
    // }
    /*data for configuration*/
    // fscanf(configuration_file_electrical, "%lf %s %s\n", &data_rate_electrical, name, unit);

    parameter_flie_electrical >> data_rate_electrical;
    parameter_flie_electrical >> name;
    parameter_flie_electrical >> unit;
    printf("%s\t%f\t%s\n", name, data_rate_electrical, unit);
    parameter_flie_electrical >> length_electrical;
    parameter_flie_electrical >> name;
    parameter_flie_electrical >> unit;
    printf("%s\t%f\t%s\n", name, length_electrical, unit);
    parameter_flie_electrical >> serdes_ratio_electrical;
    parameter_flie_electrical >> name;
    parameter_flie_electrical >> unit;
    printf("%s\t%d\t\t%s\n", name, serdes_ratio_electrical, unit);
    parameter_flie_electrical >> number_of_parallel_traces;
    parameter_flie_electrical >> name;
    parameter_flie_electrical >> unit;
    printf("%s\t\t%d\t\t%s\n", name, number_of_parallel_traces, unit);
    printf("\n");

    parameter_flie_electrical.close();
}

/*this is to write the calculated results to the file output_electrical.txt*/
void write_result_electrical() {
    printf("Electrical:\n");
    if ((output_file_electrical = fopen("output_electrical.txt", "w")) == NULL) {
        printf("Cannot open file output_electrical.txt\n");
    }

    /*writing heads*/
    fprintf(output_file_electrical, "/*********************************************************************************");
    fprintf(output_file_electrical, "\n *");
    fprintf(output_file_electrical, "\n * File name:                 output_electrical.txt");
    fprintf(output_file_electrical, "\n * Version:                   3.0");
    fprintf(output_file_electrical, "\n * Software:                  OEIL");
    fprintf(output_file_electrical, "\n * Authors:                   Zhehui Wang, Jiang Xu	");
    fprintf(output_file_electrical, "\n * Website:                   http://www.ece.ust.hk/~eexu/");
    fprintf(output_file_electrical, "\n * The copyright information of this program can be found in the file COPYRIGHT.");
    fprintf(output_file_electrical, "\n *");
    fprintf(output_file_electrical, "\n *********************************************************************************/");
    fprintf(output_file_electrical, "\n");

    /*output data written on file*/
    printf("sensitivity_la\t\t%lf\tmW\n", sensitivity_la);
    fprintf(output_file_electrical, "sensitivity_la\t\t%lf\tmW\n", sensitivity_la);
    printf("crosstalk_coefficient\t%lf\tN/A\n", crosstalk_coefficient_electrical);
    fprintf(output_file_electrical, "crosstalk_coefficient\t%lf\tN/A\n", crosstalk_coefficient_electrical);
    printf("total_attenuation\t%lf\tN/A\n", total_attenuation_electrical);
    fprintf(output_file_electrical, "total_attenuation\t%lf\tN/A\n", total_attenuation_electrical);
    printf("\t\t\t%lf\tdB\n", -10 * log10(total_attenuation_electrical));
    fprintf(output_file_electrical, "\t\t\t%lf\tdB\n", -10 * log10(total_attenuation_electrical));
    printf("energy_consumption\t%lf\tpJ/bit\n", energy_consumption_electrical);
    fprintf(output_file_electrical, "energy_consumption\t%lf\tpJ/bit\n", energy_consumption_electrical);
    printf("area_density\t\t%lf\tGbps/mm^2\n", area_density_electrical);
    fprintf(output_file_electrical, "area_density\t\t%lf\tGbps/mm^2\n", area_density_electrical);
    printf("linear_density\t\t%lf\tGbps/mm\n", linear_density_electrical);
    fprintf(output_file_electrical, "linear_density\t\t%lf\tGbps/mm\n", linear_density_electrical);
    printf("area\t\t\t%lf\tmm^2\n", area_electrical);
    fprintf(output_file_electrical, "area\t\t\t%lf\tmm^2\n", area_electrical);
    printf("latency\t\t\t%lf\tns\n", latency_value_electrical);
    fprintf(output_file_electrical, "latency\t\t\t%lf\tns\n", latency_value_electrical);
    printf("\n");
    fclose(output_file_electrical);
}

#endif /* DEFINITION_ELECTRICAL_H */