/*********************************************************************************
 *
 * File name:		defination_optical.h
 * Version:			4.0
 * Software:
 * Authors:			Zhehui Wang, Shixi Chen, Jiang Xu
 * Website:			https://eexu.home.ece.ust.hk/OEIL.html
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/
#ifndef DEFINITION_OPTICAL_H
#define DEFINITION_OPTICAL_H

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <iostream>

/*file pointer to the files*/
FILE* parameter_flie_optical;
FILE* configuration_file_optical;
FILE* output_file_optical;

/*The following is for the transmitter*/
double laser_slope_efficiency;  // the slope efficiency of laser unit:W/A
double laser_threshold_current; // the threshold current of laser unit:mA
double laser_extinction_ratio;  // the extinction ratio of laser unit:N/A
double laser_area;              // area size of laser unit:um^2
double laser_voltage;           // supply voltage of laser unit:V
double driver_voltage;          // supply voltage of the driver unit:V

/*The following is for the waveguide*/
double optical_pin_loss;    // the coupling effciecny of each optical pin unit/N/A
double optical_pin_height;  // the height of optical pin
double optical_pin_width;   // the width of optical pin
double propagation_loss;    // the propagation loss of the waveguide unit:cm^-1
double wg_refractive_index; // the group refractive index of waveguide unit:N/A
double wg_pitch;            // the pitch of waveguide

/*The following is for the receiver*/
double signal_to_noise_ratio; // signal to noise ratio unit:N/A
double tia_noise_density;     // noise density of transimpedance amplifer unit:pA/sqrt(Hz)
double tia_transimpendance;   // transimpendance of the transimpendance amplifer unit:kOhm
double la_voltage_threshold;  // threshold voltage of the limiting amplifer unit:mV
double pd_responsity;         // responsity of the photodetector unit: A/W
double pd_capacitance;        // the capacitance of the photodetector unit: fF

/*The following is for the micro-resonator*/
double mr_radius_range;     // the radius of the micro-resonator, the real length is close to the range unit:um
double mr_attenuation;      // the round trip attenuation of of the ring unit:N/A
double mr_power_split_k;    // the power split ratio that can be transmitted into the ring unit:N/A
double mr_refractive_index; // the refractive index of the mr unit:N/A
double mr_tuning_power;     // the tunning power consumption of one mr unit:mW
double mr_static_power;     // the static power consumption of one mr unit:mW
double mr_dynamic_power;    // the dynamic power consumption of one mr unit:mW/Gbps
/*The following are not inputed from "parameter.txt", they are calculated by the other parameters*/
double mr_radius_real;   // the real radius of micro-resonator unit:um
double delta_wavelength; // the spacing between two nearby wavelengths unit:nm
double mr_power_split_r; // the power split ratio that can be transmitted into the waveguide unit:N/A
double mr_area;          // the area of a single micro-resonator unit:um^2

/*The following is for serdes*/
double serdes_cur_optical;  // The unit current in SerDes circuit unit in optical interconnect unit:mA/Gbps
double serdes_area_optical; // The unit area in SerDes circuit unit in optical interconnect unit:um^2/Gbps

/*The following is for nonlinear loss model */
// #waveguide-nonlinear-loss-model#
double carrier_lifetime;
double TPA_coefficient;
double FCA_coefficient;

/*The following is for configurations*/
double data_rate_optical;  // The data rate of signals in optical interconnects, which is double the working frequency unit:N/A
double length_optical;     // the length of the optical interconnect unit:cm
int serdes_ratio_optical;  // the parallel-to-serial ratio of SerDes in optical interconnect unit:N/A
int number_of_wavelengths; // number of wavelenght in WDM optical interconnects unit:N/A
double laser_wavelength;   // the central wavelength of the laser unit: N/A
int is_direct_modulation;  // 1 for direct modulation and 0 for not modulation using MRs unit:N/A
// [nonlinear loss model]
int is_nonlinear_model_enabled;  // Set to 1 if nonlinear loss is taken into account. 0 otherwise.
double effective_mode_area;      // The effective mode area for single mode transmission in on-chip waveguide unit: cm^-2
double laser2modular_distance;   // Distance between laser source and the modular array unit: cm
double modular2coupler_distance; // Distance between the modular array and the coupler for coupling power out of the chip unit: cm
double coupler2receiver_distance; // Distance between the coupler on a second chip and the receivers on that chip unit: cm

/*The following is for calculated results*/
double sensitivity_oma;               // the sensitivity of optical receiver unit:mW
double crosstalk_coefficient_optical; // the ratio of noise power to signal power unit:N/A
double total_attenuation_optical;     // the total attenuation on the entire optical interconenct unit: N/A
double energy_consumption_optical;    // the energy consumed by interconect per bit unit:pJ/bit
double area_density_optical;          // the area bandwidth density of optical interconnect unit:Gbps/mm^2
double linear_density_optical;        // the linear bandwidth density of optical interconnect unit:Gbps/mm
double area_optical;                  // the area of optical interconnect unit:mm^2
double latency_value_optical;         // the signal latency of optical interconnect unit:ns
double optical_SNR;                   // the optical signal to noise ratio of the optical interconnect
double BER_optical;                   // the bit error rate under given signal to noise ratio


/*this is to read the paramters from the file parameter_optical.txt*/
void read_parameter_optical() {
    printf("Optical:\n");
    char name[150];
    char unit[20];
    if ((parameter_flie_optical = fopen("../config/parameter_optical.txt", "r")) == NULL) {
        printf("Cannot open file parameter_optical.txt\n");
    }
    // /*skip the text file head.*/
    // for (int i = 0; i < 37; i++) {
    //     fscanf(parameter_flie_optical, "%s\n", name);
    // }
    /*skip the text file head.*/
    for (int i = 0; i < 10; i++) {
        char* line;
        size_t len = 0;
        getline(&line, &len, configuration_file_optical);
    }
    /*parameters for transmitter*/
    fscanf(parameter_flie_optical, "%s\n", name);
    printf("%s\n", name);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &laser_slope_efficiency, name, unit);
    printf("%s\t%f\t%s\n", name, laser_slope_efficiency, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &laser_threshold_current, name, unit);
    printf("%s\t%f\t%s\n", name, laser_threshold_current, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &laser_extinction_ratio, name, unit);
    printf("%s\t%f\t%s\n", name, laser_extinction_ratio, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &laser_area, name, unit);
    printf("%s\t\t%f\t%s\n", name, laser_area, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &laser_voltage, name, unit);
    printf("%s\t\t%f\t%s\n", name, laser_voltage, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &driver_voltage, name, unit);
    printf("%s\t\t%f\t%s\n", name, driver_voltage, unit);

    /*parameters for waveguide*/
    fscanf(parameter_flie_optical, "%s\n", name);
    printf("%s\n", name);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &optical_pin_loss, name, unit);
    printf("%s\t%f\t%s\n", name, optical_pin_loss, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &optical_pin_height, name, unit);
    printf("%s\t%f\t%s\n", name, optical_pin_height, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &optical_pin_width, name, unit);
    printf("%s\t%f\t%s\n", name, optical_pin_width, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &propagation_loss, name, unit);
    printf("%s\t%f\t%s\n", name, propagation_loss, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &wg_refractive_index, name, unit);
    printf("%s\t%f\t%s\n", name, wg_refractive_index, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &wg_pitch, name, unit);
    printf("%s\t\t%f\t%s\n", name, wg_pitch, unit);

    /*parameters for receiver*/
    fscanf(parameter_flie_optical, "%s\n", name);
    printf("%s\n", name);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &signal_to_noise_ratio, name, unit);
    printf("%s\t%f\t%s\n", name, signal_to_noise_ratio, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &tia_noise_density, name, unit);
    printf("%s\t%f\t%s\n", name, tia_noise_density, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &tia_transimpendance, name, unit);
    printf("%s\t%f\t%s\n", name, tia_transimpendance, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &la_voltage_threshold, name, unit);
    printf("%s\t%f\t%s\n", name, la_voltage_threshold, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &pd_responsity, name, unit);
    printf("%s\t\t%f\t%s\n", name, pd_responsity, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &pd_capacitance, name, unit);
    printf("%s\t\t%f\t%s\n", name, pd_capacitance, unit);

    /*parameters for micro-resonator*/
    fscanf(parameter_flie_optical, "%s\n", name);
    printf("%s\n", name);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &mr_radius_range, name, unit);
    printf("%s\t\t%f\t%s\n", name, mr_radius_range, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &mr_attenuation, name, unit);
    printf("%s\t\t%f\t%s\n", name, mr_attenuation, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &mr_power_split_k, name, unit);
    printf("%s\t%f\t%s\n", name, mr_power_split_k, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &mr_refractive_index, name, unit);
    printf("%s\t%f\t%s\n", name, mr_refractive_index, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &mr_tuning_power, name, unit);
    printf("%s\t\t%f\t%s\n", name, mr_tuning_power, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &mr_static_power, name, unit);
    printf("%s\t\t%f\t%s\n", name, mr_static_power, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &mr_dynamic_power, name, unit);
    printf("%s\t%f\t%s\n", name, mr_dynamic_power, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &mr_area, name, unit);
    printf("%s\t\t\t%f\t%s\n", name, mr_area, unit);

    /*parameters for serdes*/
    fscanf(parameter_flie_optical, "%s\n", name);
    printf("%s\n", name);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &serdes_cur_optical, name, unit);
    printf("%s\t%f\t%s\n", name, serdes_cur_optical, unit);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &serdes_area_optical, name, unit);
    printf("%s\t%f\t%s\n", name, serdes_area_optical, unit);

    // #waveguide-nonlinear-loss-model#
    // 4e-9        carrier_lifetime        s
    // 8e-10       TPA_coefficient     cm/W
    // 1.45e-17    FCA_coefficient     cm^2
    fscanf(parameter_flie_optical, "%s\n", name);
    printf("%s\n", name);
    fscanf(parameter_flie_optical, "%lf %s %s\n", &carrier_lifetime, name, unit);
    std::cout << name << "\t" << carrier_lifetime << "\t\t" << unit << std::endl;
    fscanf(parameter_flie_optical, "%lf %s %s\n", &TPA_coefficient, name, unit);
    std::cout << name << "\t\t" << TPA_coefficient << "\t\t" << unit << std::endl;
    fscanf(parameter_flie_optical, "%lf %s %s\n", &FCA_coefficient, name, unit);
    std::cout << name << "\t\t" << FCA_coefficient << "\t" << unit << std::endl;

    printf("\n\n");
    fclose(parameter_flie_optical);
}

/*this is to read the confiuration from the file configuration_optical.txt*/
void read_configuration_optical() {
    printf("Optical:\n");
    char name[150];
    char unit[120];
    if ((configuration_file_optical = fopen("../config/configuration_optical.txt", "r")) == NULL) {
        printf("Cannot open file configuration_optical.txt\n");
    }
    // for (int i = 0; i < 10; i++) {
    //     std::cout << "Here is the " << i << "th time we enter this loop." << std::endl;
    //     printf("%dth time reading config.\n", i);
    //     fscanf(configuration_file_optical, "%s\n", name);
    //     printf("What???? %s Are ya kidding?\n", name);
    // }
    /*skip the text file head.*/
    for (int i = 0; i < 10; i++) {
        char* line;
        size_t len = 0;
        getline(&line, &len, configuration_file_optical);
    }
    /*data for configuration*/
    fscanf(configuration_file_optical, "%lf\t%s\t%s\n", &data_rate_optical, name, unit);
    printf("%s\t\t%lf\t%s\n", name, data_rate_optical, unit);
    fscanf(configuration_file_optical, "%lf %s %s\n", &length_optical, name, unit);
    printf("%s\t\t\t%lf\t%s\n", name, length_optical, unit);
    fscanf(configuration_file_optical, "%d %s %s\n", &serdes_ratio_optical, name, unit);
    printf("%s\t\t%d\t\t%s\n", name, serdes_ratio_optical, unit);
    fscanf(configuration_file_optical, "%d %s %s\n", &number_of_wavelengths, name, unit);
    printf("%s\t\t%d\t\t%s\n", name, number_of_wavelengths, unit);
    fscanf(configuration_file_optical, "%lf %s %s\n", &laser_wavelength, name, unit);
    printf("%s\t\t%lf\t%s\n", name, laser_wavelength, unit);
    fscanf(configuration_file_optical, "%d %s %s\n", &is_direct_modulation, name, unit);
    std::cout << name << "\t\t" << std::boolalpha << (bool)is_direct_modulation << "\t\t" << unit << std::endl;
    // printf("%s\t\t%d\t\t%s\n", name, is_direct_modulation, unit);

    // Nonlinear loss model
    // 0       is_nonlinear_model_enabled        N/A
    // 1e-8        effective_mode_area     cm^-2
    // 0.5     laser2modular_distance        cm
    // 0.5     modular2coupler_distance        cm
    // 0.5     coupler2receiver_distance        cm
    fscanf(configuration_file_optical, "%d %s %s\n", &is_nonlinear_model_enabled, name, unit);
    std::cout << name << "\t" << std::boolalpha << (bool)is_nonlinear_model_enabled << "\t\t" << unit << std::endl;
    // printf("%s\t%d\t\t%s\n", name, is_nonlinear_model_enabled, unit);
    fscanf(configuration_file_optical, "%lf %s %s\n", &effective_mode_area, name, unit);
    printf("%s\t\t%lf\t%s\n", name, effective_mode_area * 1e8, "um^2");
    fscanf(configuration_file_optical, "%lf %s %s\n", &laser2modular_distance, name, unit);
    printf("%s\t\t%lf\t%s\n", name, laser2modular_distance, unit);
    fscanf(configuration_file_optical, "%lf %s %s\n", &modular2coupler_distance, name, unit);
    printf("%s\t%lf\t%s\n", name, modular2coupler_distance, unit);
    fscanf(configuration_file_optical, "%lf %s %s\n", &coupler2receiver_distance, name, unit);
    printf("%s\t%lf\t%s\n", name, coupler2receiver_distance, unit);
    printf("\n");
    fclose(configuration_file_optical);
}

/*this is to write the calculated results to the file output.txt*/
void write_result_optical() {
    printf("Optical:\n");
    if ((output_file_optical = fopen("output_optical.txt", "w")) == NULL) {
        printf("Cannot open file output_optical.txt\n");
    }
    /*writing heads*/
    fprintf(output_file_optical, "/*********************************************************************************");
    fprintf(output_file_optical, "\n *");
    fprintf(output_file_optical, "\n * File name:                 output_optical.txt");
    fprintf(output_file_optical, "\n * Version:                   3.0");
    fprintf(output_file_optical, "\n * Software:                  OEIL");
    fprintf(output_file_optical, "\n * Authors:                   Zhehui Wang, Jiang Xu	");
    fprintf(output_file_optical, "\n * Website:                   http://www.ece.ust.hk/~eexu/");
    fprintf(output_file_optical, "\n * The copyright information of this program can be found in the file COPYRIGHT.");
    fprintf(output_file_optical, "\n *");
    fprintf(output_file_optical, "\n *********************************************************************************/");
    fprintf(output_file_optical, "\n");

    /*output data written on file*/
    printf("sensitivity_oma\t\t%lf\tmW\n", sensitivity_oma);
    fprintf(output_file_optical, "sensitivity_oma\t\t%lf\tmW\n", sensitivity_oma);
    printf("crosstalk_coefficient\t%lf\tN/A\n", crosstalk_coefficient_optical);
    fprintf(output_file_optical, "crosstalk_coefficient\t%lf\tN/A\n", crosstalk_coefficient_optical);
    printf("total_attenuation\t%lf\tN/A\n", total_attenuation_optical);
    fprintf(output_file_optical, "total_attenuation\t%lf\tN/A\n", total_attenuation_optical);
    printf("\t\t\t%lf\tdB\n", -10 * log10(total_attenuation_optical));
    fprintf(output_file_optical, "\t\t\t%lf\tdB\n", -10 * log10(total_attenuation_optical));
    printf("energy_consumption\t%lf\tpJ/bit\n", energy_consumption_optical);
    fprintf(output_file_optical, "energy_consumption\t%lf\tpJ/bit\n", energy_consumption_optical);
    printf("area_density\t\t%lf\tGbps/mm^2\n", area_density_optical);
    fprintf(output_file_optical, "area_density\t\t%lf\tGbps/mm^2\n", area_density_optical);
    printf("linear_density\t\t%lf\tGbps/mm\n", linear_density_optical);
    fprintf(output_file_optical, "linear_density\t\t%lf\tGbps/mm\n", linear_density_optical);
    printf("area\t\t\t%lf\tmm^2\n", area_optical);
    fprintf(output_file_optical, "area\t\t\t%lf\tmm^2\n", area_optical);
    printf("latency\t\t\t%lf\tns\n", latency_value_optical);
    fprintf(output_file_optical, "latency\t\t\t%lf\tns\n", latency_value_optical);
    printf("optical_SNR\t\t%lf\tdB\n", optical_SNR);
    fprintf(output_file_optical, "optical_SNR\t\t%lf\tN/A\n", optical_SNR);
    printf("BER_optical\t\t%e\tNA\n", BER_optical);
    fprintf(output_file_optical, "BER_optical\t\t%lf\tN/A\n", BER_optical);
    printf("\n");

    fclose(output_file_optical);
}

#endif